.list
.listmac
.include "tn44adef.inc"		; Definition file for Tiny44A

//  Standard thresholds
.equ inpThrRed = int(1.1*256/4.1)	; 0.2v scaled to ref
.equ inpThrGrn = int(1.2*256/4.1)	; 0.3v scaled to ref
.equ inpThrBlu = int(1.3*256/4.1)	; 0.4v scaled to ref
/*  Special case for reworked with wrong LEDs
.equ inputThresh1 = int(1.1*256/4.1)	; 0.2v scaled to ref
.equ inputThresh2 = int(1.25*256/4.1)	; 0.3v scaled to ref
.equ inputThresh3 = int(1.15*256/4.1)	; 0.4v scaled to ref
*/
.equ inToOutDelay = 300			; ms from input change to output change
.equ redLED = PB0				; Output pins for LEDs
.equ grnLED = PB1
.equ bluLED = PB2
.equ ADC0Pin = PA0
.equ ADC1Pin = PA1
.equ ADC2Pin = PA2
.equ ADC3Pin = PA3
.equ ADC4Pin = PA4
.equ ADC5Pin = PA5
.equ ADC6Pin = PA6
.equ ADC7Pin = PA7
.equ redInChan = 0				; ADC channels that correspond to sensor LEDs
.equ grnInChan = 2
.equ bluInChan = 3
.equ redInpDis = ADC0D			; Input disable for ADC pins in use
.equ grnInpDis = ADC2D
.equ bluInpDis = ADC3D
.equ ADCchanMask = 0x07			; Selects the channel setting
.equ ADCnonChanMask = 0xF8		; Selects all but channel setting
.equ ARefVcc = 0				; Use Vcc as ref for ADC
.equ ARefExt = 1				; Use AREF (PA0) as ref for ADC
.equ ARef11 = 2					; Use int 1.1v as ref for ADC


.cseg	
.org 0x0000						; Starts at location 0000
	rjmp	RESETisr			; Reset
	rjmp	EXT_INT0isr			; External Interrupt 0
	rjmp	PCINT0isr			; Pin change Interrupt Request 0
	rjmp	PCINT1isr			; Pin change Interrupt Request 1
	rjmp	WDTisr				; Watchdog Interrupt Handler
	rjmp	TIM1_CAPTisr		; Timer1 Capture Handler
	rjmp	TIM1_COMPAisr		; Timer/Counter1 Compare Match A
	rjmp	TIM1_COMPBisr		; Timer/Counter1 Compare Match B
	rjmp	TIM1_OVFisr			; Timer/Counter1 Overflow
	rjmp	TIM0_COMPAisr		; Timer/Counter0 Compare Match A
	rjmp	TIM0_COMPBisr		; Timer/Counter0 Compare Match B
	rjmp	TIM0_OVFisr			; Timer/Counter0 Overflow
	rjmp	ANA_COMPisr			; Analog comparator
	rjmp	ADCisr				; ADC Conversion ready
	rjmp	EE_RDYisr			; EEPROM Ready
	rjmp	USI_STRisr			; USI START
	rjmp	USI_OVFisr			; USI Overflow

RESETisr:
/*		Note clock default is 8 MHz, 14*CK+64ms start-up, CK/8
			=> System Clock = 1 MHz

		Initialization Section
*/
	ldi		r16, high(RAMEND)			; Main program start
	out		SPH,r16						; Set Stack Pointer to top of RAM
	ldi		r16, low(RAMEND)
	out		SPL,r16							
;
; Turn off unused features
										; Shut down timer 1 and USI to save power
//	ldi		r16,(1<<PRTIM1)+(1<<PRTIM0)+(1<<PRUSI)+(1<<PRADC)
	ldi		r16,(1<<PRTIM1)+(1<<PRUSI)
	out 	PRR, r16
;
; Set up digital outputs
										; Set port A as input pins
	clr		r16
	out 	DDRA, r16
										; Set port A inputs to tri-state
	clr		r16
										; Set port A inputs to pull-ups if PU enabled
//	ldi		r16,(1<<PA0)+(1<<PA1)+(1<<PA2)+(1<<PA3)+(1<<PA4)+(1<<PA5)+(1<<PA6)+(1<<PA7)
	out		PortA, r16
;
										; Set port B as output pins
	ldi		r16,(1<<redLED)+(1<<grnLED)+(1<<bluLED)
	out 	DDRB, r16
										; Set port B outputs high (LEDs off)
//	ldi 	r16, 0x00	; all LEDs on
	out		PortB, r16
;
; Setup Timer 0
	ldi		r16, (1<<WGM01)				; CTC mode 2 clears Timer 0 on match, outputs disconnected
	out		TCCR0A, r16

	ldi		r16,(1<<CS02)+(1<<CS00)		; Set timer 0 prescale to 1024 (1 ms count)
	out		TCCR0B, r16

	ldi		r16, 244					; Set T0 compare reg A to 244 (4 Hz)
	out		OCR0A, r16


	ldi		r16,(1<<OCIE0A)				; Enable Timer 0 compare A interrupt
	out		TIMSK0, r16

	ldi		r16, (1<<PSR10)				; Set timer sync mode, start timer
	out		GTCCR, r16
;
; Setup ADC
//										; Set up ADC ref Vcc, chan Red input
//	ldi		r16,redInChan
										; Set up ADC ref 1.1v, chan Red input
	ldi		r16,ARefVcc+redInChan
	out 	ADMUX, r16

	ldi		r16, redInChan				; First two reads will be channel 0 in free-run mode
	sts		currentChan, r16
	sts		nextChan, r16
										; Left-adjust result, free running mode
	ldi		r16,(1<<ADLAR)
	out 	ADCSRB, r16
										; Disable digital inputs on ADC pins in use
	ldi		r16,(1<<redInpDis)+(1<<grnInpDis)+(1<<bluInpDis)
	out 	DIDR0, r16
										; Enable ADC, start first conv, enable int, clk/8 pre-scale
										; Set for free-running conversion
										; This means each ADC sample will take at most 104 clock cycles
//	ldi		r16,(1<<ADEN)+(1<<ADSC)+(1<<ADATE)+(1<<ADIE)+(1<<ADPS1)+(1<<ADPS0)

										; Enable ADC, start first conv, enable int, clk/32 pre-scale
										; Single sample mode
										; This means each ADC sample will take at most 104 clock cycles
	ldi		r16,(1<<ADEN)+(1<<ADSC)+(1<<ADIE)+(1<<ADPS2)

										; Enable ADC, start first conv, enable int, clk/16 pre-scale
										; This means each ADC sample will take at most 208 clock cycles
//	ldi		r16,(1<<ADEN)+(1<<ADSC)+(1<<ADATE)+(1<<ADIE)+(1<<ADPS2)
	out 	ADCSRA, r16
;
; End of initialization
;
	ldi		r16, (1<<SE)				; Allow SLEEP mode = Idle
	out		MCUCR, r16
	sei									; Enable interrupts

/*
		Core program: (currently does nothing)
*/
Main:
	sleep								; Main routine would go here
	rjmp 	Main						; Re-run loop
/*
		Interrupt Service Routines
*/
EXT_INT0isr:							; External Interrupt 0
	reti
PCINT0isr:								; Pin change Interrupt Request 0
	reti
PCINT1isr:								; Pin change Interrupt Request 1
	reti
WDTisr:									; Watchdog Time-out
	reti
TIM0_COMPAisr:							; Timer/Counter0 Compare Match A
/*
		Timer 0 is used to set the delay time between when an input is sensed and
		when the output is updated.  The prescaler is set for 1 ms ticks of the timer.

*/
	push	r16							; Save contents of temp regs
	push	r17
	in		r16,SREG					; Get status register
	push	r16							; Save on the stack
//
/*	lds		r16, countDown+redInChan	; Decrement the timers for each output
	tst		r16
	breq	countDownGrn				; If 0, skip to next output timer
	dec		r16
	sts		countDown+redInChan, r16	; Save new value
	brne	countDownGrn				; If just new value = 0, then update the output
	lds		r16, inputFlag+redInChan	; Check what the state should be
	tst		r16
	brne	turnRedOff					; If flag is 0 (turn LED OFF), then jump
//	cbi		PortB, redLED				; Output low turns LED on
	rjmp	countDownGrn
turnRedOff:
//	sbi		PortB, redLED				; Output high turns LED off
countDownGrn:
	lds		r16, countDown+grnInChan	; Decrement the timers for each output
	tst		r16
	breq	countDownBlu				; If 0, skip to next output timer
	dec		r16
	sts		countDown+grnInChan, r16	; Save new value
	brne	countDownBlu				; If just new value = 0, then update the output
	lds		r16, inputFlag+grnInChan	; Check what the state should be
	tst		r16
	brne	turnGrnOff					; If flag is 0 (turn LED OFF), then jump
//	cbi		PortB, grnLED				; Output low turns LED on
	rjmp	countDownBlu
turnGrnOff:
//	sbi		PortB, grnLED				; Output high turns LED off
countDownBlu:
	lds		r16, countDown+bluInChan	; Decrement the timers for each output
	tst		r16
	breq	countDownEnd				; If 0, skip to next output timer
	dec		r16
	sts		CountDown+bluInChan, r16	; Save new value
	brne	countDownEnd				; If just new value = 0, then update the output
	lds		r16, inputFlag+bluInChan	; Check what the state should be
	tst		r16
	brne	turnBluOff					; If flag is 0 (turn LED OFF), then jump
//	cbi		PortB, bluLED				; Output low turns LED on
	rjmp	countDownEnd
turnBluOff:
//	sbi		PortB, bluLED				; Output high turns LED off
countDownEnd:
*/
	pop		r16							; Restore status register
	out		SREG,r16
	pop		r17							; Restore other regs
	pop		r16
	reti
;
TIM0_COMPBisr:							; Timer/Counter0 Compare Match B
	reti
TIM0_OVFisr:							; Timer/Counter0 Overflow
	reti
TIM1_CAPTisr:							; Timer1 Capture Handler
	reti
TIM1_COMPAisr:							; Timer/Counter1 Compare Match A
	reti
TIM1_COMPBisr:							; Timer/Counter1 Compare Match B
	reti
TIM1_OVFisr:							; Timer/Counter1 Overflow
	reti
ANA_COMPisr:							; Analog comparator
	reti
ADCisr:									; ADC Conversion ready
/*	Three of the ADC channels are in use.  The ADC is set for free-run
	mode, so this routine gets the value and stores it into the input array, then
	keeps track of pointers and updates the multiplexer for the next reading.  Note
	that because of the ADC timing, the multiplexer value set now will determine the
	channel for the reading after next.
	The ADC scale is 1 bit = 1/256 of Vcc.  Each color has a differnet threshhold.
*/
// Save machine state of registers that will be used in this ISR
	in		r31,SREG					; Get status register
//
// Get value from latest conversion
	in		r28, ADCH					; Get 8-bit left-justified value
	in		r29, ADMUX					; Get channel for this sample
	andi	r29, ADCchanMask
tryRed:
	cpi		r29, redInChan				; Determine source of current reading
	brne	tryGrn
	sts		ADCvalue+redInChan, r28		; Save in value array
	cpi		r28, inpThrRed				; Check if input activated and set input flag
	brlo	redOff
redOn:
	cbi		PortB, redLED				; Output low turns LED on
	rjmp	redDone
redOff:
	sbi		PortB, redLED				; Output high turns LED off
redDone:
	ldi		r28, grninChan
	rjmp	setMUXChan
tryGrn:
	cpi		r29, grnInChan				; Determine source of current reading
	brne	tryBlu
	sts		ADCvalue+grnInChan, r28		; Save in value array
	cpi		r28, inpThrGrn				; Check if input activated and set input flag
	brlo	grnOff
grnOn:
	cbi		PortB, grnLED				; Output low turns LED on
	rjmp	grnDone
grnOff:
	sbi		PortB, grnLED				; Output high turns LED off
grnDone:
	ldi		r28, bluInChan
	rjmp	setMUXChan
tryBlu:
	cpi		r29, bluInChan				; Determine source of current reading
	brne	invalidChan
	sts		ADCvalue+bluInChan, r28		; Save in value array
	cpi		r28, inpThrBlu				; Check if input activated and set input flag
	brlo	bluOff
bluOn:
	cbi		PortB, bluLED				; Output low turns LED on
	rjmp	bluDone
bluOff:
	sbi		PortB, bluLED				; Output high turns LED off
bluDone:
	ldi		r28, redInChan
	rjmp	setMUXChan
invalidChan:
	ldi		r28, redInChan
setMuxChan:
	ldi		r30, ARefVcc
	add		r30, r28
	out		ADMUX, r30					; Set the MUX to the new channel
										; Start next conversion
//	sbi		ADCSRA, ADSC
	ldi		r30,(1<<ADEN)+(1<<ADSC)+(1<<ADIE)+(1<<ADPS2)+(1<<ADPS0)
	out		ADCSRA, r30
restoreState:
	out		SREG,r31					; Restore status register
	reti								; Done with ISR
/*
//	Test routine that alternates red/blue with green.
//	Duty cycle depends on voltage level using counter
	in		r28, ADCH					; Get 8-bit left-justified value
	cpi		r28, inputThresh3			; Check if input activated and set input flag
	cp		r28, r30					; Compare to ADC cycle counter
	brlo	redOff
	cbi		PortB, redLED				; Output low turns LED on
	sbi		PortB, grnLED				; Output high turns LED off
	cbi		PortB, bluLED				; Output low turns LED on
	rjmp	redDone
redOff:
	sbi		PortB, redLED				; Output high turns LED off
	cbi		PortB, grnLED				; Output low turns LED on
	sbi		PortB, bluLED				; Output high turns LED off
RedDone:
	in		r28, ADCSRA
	ori		r28, (1<<ADSC)
	out		ADCSRA, r28
	dec		r30							; Decrement ADC cycle counter
*/
/*
// Get value from latest conversion
	in		r28, ADCH					; Get 8-bit left-justified value
	lds		r29, currentChan			; Get channel for this sample
tryRed:
	cpi		r29, redInChan				; Determine source of current reading
	brne	tryGrn
	sts		ADCvalue+redInChan, r28		; Save in value array
	cpi		r28, inputThresh1			; Check if input activated and set input flag
	lds		r28, inputFlag+redInChan	; Get prev input flag state
	brlo	redOff
	cbi		PortB, redLED				; Output low turns LED on
	cpi		r28, 0xFF
	breq	redDone						; If not changed, skip to end
	lds		r28, inToOutDelay
	sts		CountDown+redInChan, r28	; else store countdown timer value
	ser		r28							; new value for output flag
	rjmp	storeRedFlag
redOff:
	sbi		PortB, redLED				; Output high turns LED off
	cpi		r28, 0x00
	breq	readDone					; If not changed, skip to end
	lds		r28, inToOutDelay
	sts		CountDown+redInChan, r28	; else store countdown timer value
	clr		r28							; new value for output flag
storeRedFlag:
	sts		inputFlag+redInChan, r28	; Save new value of output flag
redDone:
	rjmp	readDone
tryGrn:
	cpi		r29, grnInChan				; Determine source of current reading
	brne	tryBlu
	sts		ADCvalue+grnInChan, r28		; Save in value array
	cpi		r28, inputThresh2			; Check if input activated and set input flag
	lds		r28, inputFlag+grnInChan	; Get current flag value
	brlo	grnOff						; If input below threshold, LED should be off
	cbi		PortB, grnLED				; Turn green LED on
	cpi		r28, 0xFF					; Check if input changed	
	breq	readDone					; If not, do nothing
	lds		r28, inToOutDelay			; Otherwise, start delay timer
	sts		CountDown+grnInChan, r28
	ser		r28							; Set the new flag value to replace current
	rjmp	storeGrnFlag
grnOff:
	sbi		PortB, grnLED				; Output high turns LED off
	cpi		r28, 0x00					; Check if input changed
	breq	readDone					; If not, do nothing
	lds		r28, inToOutDelay			; Otherwise, start delay timer
	sts		CountDown+grnInChan, r28
	clr		r28
storeGrnFlag:
	sts		inputFlag+grnInChan, r28
	rjmp	readDone
tryBlu:
	cpi		r29, bluInChan				; Determine source of current reading
	brne	readDone
	sts		ADCvalue+bluInChan, r28		; Save in value array
	cpi		r28, inputThresh3			; Check if input activated and set input flag
	lds		r28, inputFlag+bluInChan
	brlo	bluOff
	cbi		PortB, bluLED				; Output low turns LED on
	cpi		r28, 0xFF
	breq	readDone
	lds		r28, inToOutDelay
	sts		CountDown+bluInChan, r28
	ser		r28
	rjmp	storeBluFlag
bluOff:
	sbi		PortB, bluLED				; Output high turns LED off
	cpi		r28, 0x00
	breq	readDone
	lds		r28, inToOutDelay
	sts		CountDown+bluInChan, r28
	clr		r28
storeBluFlag:
	sts		inputFlag+bluInChan, r28
readDone:
// Update the channel ptr for the next sample
	in		r28, ADMUX					;
	andi	r28, ADCchanMask			; Get channel number mux is now set to
	sts		currentChan, r28			; Save to current pointer
// Set up mux for next sampling
	cpi		r28, redInChan				; If it's red, green is next
	brne	nextChan1
	ldi		r28, grninChan
	rjmp	setMuxChan
nextChan1:
	cpi		r28, grnInChan				; If it's green, blue is next
	brne	nextChan2
	ldi		r28, bluInChan
	rjmp	setMuxChan
nextChan2:
	cpi		r28, bluInChan				; If it's blue, red is next
	brne	restoreState
	ldi		r28, redInChan
setMuxChan:
	in		r29, ADMUX
	andi	r29, ADCnonChanMask			; Get all but channel
	or		r29, r28
	out		ADMUX, r29					; Set the MUX to the new channel
	sts		nextChan, r28				; move next channel ptr value to current pointer
// Restore machine state
restoreState:
	out		SREG,r31					; Restore status register
	reti								; Done with ISR
*/
;
EE_RDYisr:								; EEPROM Ready
	reti
USI_STRisr:								; USI START
	reti
USI_OVFisr:								; USI Overflow
	reti

.dseg							// Static RAM Area
.org		SRAM_START			; Skip over the memory-mapped registers
Timer0:		.byte 1				; Byte for extending timer 0 to 16 bits
T0Flag:		.byte 1
currentChan:.byte 1				; Points to ADC input current reading will be from
nextChan:	.byte 1				; Points to next ADC input already in MUX setting
ADCvalue:	.byte 8				; Array of input readings from the 8 ADC channels
inputFlag:	.byte 8				; Array of flags indicating if input is over threshold
countDown:	.byte 8				; Array of counters for output changes

.eseg							// EEPROM Area
